/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import java.io.Serializable;


/**
 * <p>
 * A structure for holding url and the page title of node. The url should have the query string decoded.
 * </p>
 *
 * <p>
 * <b>Implementation notes:</b> This structure will be created by the NodeDiscovery (when discovering the current node)
 * and by the PathDiscovery implementations to return the path from the current node to the root node.  Each node in
 * that path will be represented by this class.
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> This class is immutable and thread safe.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public class BreadCrumbNode implements Serializable {
    /**
     * <p>
     * Represents the title associated with this node.  This variable is set by the constructor and is immutable.  The
     * value can be null if the title should default to the title found in the PathDiscovery.  The value should not be
     * null if it's a node that is part of a path returned by the PathDiscovery implementation.  This variable is
     * referenced by the associated getter.
     * </p>
     */
    private final String title;

    /**
     * <p>
     * Represents the url (including query string) associated with this node.  This variable is set by the constructor,
     * is immutable and will never be null.  This variable is referenced by the associated getter.
     * </p>
     */
    private final String url;

    /**
     * <p>
     * Constructor for BreadCrumbNode from the specified fields.
     * </p>
     *
     * @param url A non-null, non-empty (trim'd) url of page represented by this node
     * @param title A possibly null, possibly empty (trim'd) title of page represented by this node
     *
     * @throws IllegalArgumentException if url is null or an empty (trim'd) string
     */
    public BreadCrumbNode(String url, String title) {
        if (url == null) {
            throw new IllegalArgumentException("url is null");
        }
        if (url.trim().length() == 0) {
            throw new IllegalArgumentException("url is empty string");
        }
        this.url = url;
        this.title = ((title != null) && (title.trim().length() == 0)) ? null : title;
    }

    /**
     * <p>
     * Simply the getter method for the title of the node.
     * </p>
     *
     * @return A possibly null, non-empty (trim'd) string representing the title of the node
     */
    public String getTitle() {
        return title;
    }

    /**
     * <p>
     * Simply the getter method for the url of the node.  The returned string will be the URL with any query string.
     * </p>
     *
     * @return A non-null, non-empty (trim'd) string representing the url (including query string) of the node
     */
    public String getUrl() {
        return url;
    }
}
