/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.ConfigManagerException;
import com.topcoder.util.objectfactory.ObjectFactory;
import com.topcoder.util.objectfactory.impl.ConfigManagerSpecificationFactory;

import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.XmlPathDiscovery;

import java.io.IOException;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.ListIterator;

import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.BodyTagSupport;


/**
 * <p>
 * The main tag class that application will use. This iteration tag will render the current node to the root node (or
 * vice-versa) using a path separator and specific styles.
 * </p>
 *
 * <p>
 * The object factory will be used to discover the node and the path to the root. The node discovery is optional. If
 * the node discovery is not specified, it is assumed a node discovery tag was used on the page to define the current
 * node under the BREADCRUMB_CURRENT_NODE attribute. The body of the tag will be used to format the various trails,
 * using scripting variables defined by this class, namely, BREADCRUMB_FORMAT_NODE and BREADCRUMB_FORMAT_STYLE_CLASS.
 * </p>
 *
 * <p>
 * The application can setup the separator and the various styles either by specifying default values in the
 * configuration manager or by specifying attributes of the tag. The configuration namespace to use is
 * "com.topcoder.web.ui.tag.breadcrumbtrail". Here are the supported attributes, which are also configurable:
 *
 * <ul>
 * <li>
 * <b>pathSeparator</b> (optional) - the separator that will be drawn between nodes in a path. Default is ":".
 * </li>
 * <li>
 * <b>pathSeparatorStyle</b> (optional) - the separator CSS style that will be used to draw the separator. Default is
 * to use the style that is currently in context.
 * </li>
 * <li>
 * <b>nodeStyle</b> (optional) - the node (non-root and non-current) CSS style that will be used to draw the node's
 * title. Default is to use the style that is currently in context.
 * </li>
 * <li>
 * <b>currentNodeStyle</b> (optional) - the current (only) node CSS style that will be used to draw the node's title.
 * Default is to use the style that is currently in context.
 * </li>
 * <li>
 * <b>rootNodeStyle</b> (optional) - the root (only) node CSS style that will be used to draw the node's title. Default
 * is to use the style that is currently in context.
 * </li>
 * <li>
 * <b>mouseOverStyle</b> (optional) - the mouse over node CSS style that will be used to draw the node's title. Default
 * is to use the style that is currently in context.
 * </li>
 * <li>
 * <b>silentErrors</b> (optional) - whether any exceptions should be silently ignored or not. Default is false.
 * </li>
 * <li>
 * <b>pathDirection</b> (optional) - the direction of the path.  The current allowable values are either
 * ROOT_TO_CURRENT (specifying root [on the left] to current [on the right]) or  CURRENT_TO_ROOT (specifying current
 * [on the left] to root [on the right]). Default is ROOT_TO_CURRENT.
 * </li>
 * </ul>
 *
 * Here are the supported attributes, which are not configurable:
 *
 * <ul>
 * <li>
 * <b>pathOverride</b> (optional) - the path override for this specific tag to be used in XmlPathDiscovery as a direct
 * XML string. If not specified, normal path discovery should be used.
 * </li>
 * <li>
 * <b>title</b> (optional) - the title that will be used if the node is being discovered by this tag (i.e. no other tag
 * has already discovered the node and set the BREADCRUMB_CURRENT_NODE attribute). If not specified, the title should
 * default to the title specified in the PathDiscovery data source.
 * </li>
 * </ul>
 *
 * Note that when the tag is reused, the attributes should default to the settings loaded from the configuration.
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> This class is thread safe in the context of the JSP rendering process.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public class BreadCrumbTrailTag extends BodyTagSupport {
    /**
     * <p>
     * The configuration namespace to use to load the values of the common attributes.
     * </p>
     */
    private static final String NAMESPACE = "com.topcoder.web.ui.tag.breadcrumbtrail";

    /**
     * <p>
     * The property key specifying the separator that will be drawn between nodes in a path. This is optional.
     * </p>
     */
    private static final String PATH_SEPARATOR_PROPERTY = "pathSeparator";

    /**
     * <p>
     * The property key specifying the separator CSS style that will be used to draw the separator. This is optional.
     * </p>
     */
    private static final String PATH_SEPARATOR_STYLE_PROPERTY = "pathSeparatorStyle";

    /**
     * <p>
     * The property key specifying the node (non-root and non-current) CSS style that will be used to draw the node's
     * title. This is optional.
     * </p>
     */
    private static final String NODE_STYLE_PROPERTY = "nodeStyle";

    /**
     * <p>
     * The property key specifying the current (only) node CSS style that will be used to draw the node's title. This
     * is optional.
     * </p>
     */
    private static final String CURRENT_NODE_STYLE_PROPERTY = "currentNodeStyle";

    /**
     * <p>
     * The property key specifying the root (only) node CSS style that will be used to draw the node's title. This is
     * optional.
     * </p>
     */
    private static final String ROOT_NODE_STYLE_PROPERTY = "rootNodeStyle";

    /**
     * <p>
     * The property key specifying the mouse over node CSS style that will be used to draw the node's title. This is
     * optional.
     * </p>
     */
    private static final String MOUSE_OVER_STYLE_PROPERTY = "mouseOverStyle";

    /**
     * <p>
     * The property key specifying whether any exceptions should be silently ignored or not. This is optional.
     * </p>
     */
    private static final String SILENT_ERRORS_PROPERTY = "silentErrors";

    /**
     * <p>
     * The property key specifying the direction of the path. This is optional.
     * </p>
     */
    private static final String PATH_DIRECTION_PROPERTY = "pathDirection";

    /**
     * <p>
     * The CSS style class specifying the separator CSS style that will be used to draw the separator.
     * </p>
     */
    private static final String PATH_SEPARATOR_STYLE = "path-separator-style";

    /**
     * <p>
     * The CSS style class specifying the node (non-root and non-current) CSS style that will be used to draw the
     * node's title.
     * </p>
     */
    private static final String NODE_STYLE = "node-style";

    /**
     * <p>
     * The CSS style class specifying the current (only) node CSS style that will be used to draw the node's title.
     * </p>
     */
    private static final String CURRENT_NODE_STYLE = "current-node-style";

    /**
     * <p>
     * The CSS style class specifying the root (only) node CSS style that will be used to draw the node's title.
     * </p>
     */
    private static final String ROOT_NODE_STYLE = "root-node-style";

    /**
     * <p>
     * The CSS sub-style specifying the CSS style that will be used to render a link.
     * </p>
     */
    private static final String LINK_STYLE = "a:link";

    /**
     * <p>
     * The CSS sub-style specifying the CSS style that will be used to define the behavior when the mouse is over a
     * link.
     * </p>
     */
    private static final String HOVER_STYLE = "a:hover";

    /**
     * <p>
     * This static constant represents the page scope attribute name that the current node, if found, would be found
     * under.  This constant will be used by an AbstractNodeDiscovery instance to store the current node in the
     * attribute map and by this class to retrieve.
     * </p>
     */
    public static final String BREADCRUMB_CURRENT_NODE = "breadCrumbCurrentNode";

    /**
     * <p>
     * This static constant represents the nested variable name that the current node to format would be found under.
     * This constant will be used by this class to define a scripting variable available to a node formatter to
     * format.  Note: this is the page scope attribute name that will be made available as a variable via the
     * BreadCrumbTrailTagExtraInfo class.
     * </p>
     */
    public static final String BREADCRUMB_FORMAT_NODE = "breadCrumbFormatNode";

    /**
     * <p>
     * This static constant represents the nested variable name that the current node's style that  will be used to
     * format.  This constant will be used by this class to define a scripting variable available to a node formatter
     * to format.  Note: this is the page scope attribute name that will be made available as a variable via the
     * BreadCrumbTrailTagExtraInfo class.
     * </p>
     */
    public static final String BREADCRUMB_FORMAT_STYLE_CLASS = "breadCrumbFormatStyleClass";

    /**
     * <p>
     * This static constant represents a "pathDirection" constant for formatting the path from the root (on the left)
     * to the current node (on the right).  This variable is only referenced in the doStartTag() but is public for a
     * convenience to strategy implementations.
     * </p>
     */
    public static final int ROOT_TO_CURRENT = 0;

    /**
     * <p>
     * This static constant represents a "pathDirection" constant for formatting the path from the current node (on the
     * left) to the root node (on the right).  This variable is only referenced in the doStartTag() but is public for
     * a convenience to strategy implementations.
     * </p>
     */
    public static final int CURRENT_TO_ROOT = 1;

    /**
     * <p>
     * Represents the separator that will be drawn between nodes in a path.   This variable is set by the constructor,
     * is mutable and will never be null but can be an empty (trim'd) string. This variable is referenced by the
     * associated getter/setter methods and the doAfterBody() method.
     * </p>
     */
    private String pathSeparator;

    /**
     * <p>
     * Represents the configurable separator that will be drawn between nodes in a path.   This variable is set by the
     * constructor, is immutable and will never be null but can be an empty (trim'd) string.
     * </p>
     */
    private final String configPathSeparator;

    /**
     * <p>
     * Represents the separator CSS style that will be used to draw the separator.  This is the actual CSS style that
     * will be rendered to the underlying text. This variable is set by the constructor, is mutable and can be null to
     * specify no specific style (i.e. use the style that is currently in context). This variable is referenced by the
     * associated getter/setter methods and the doAfterBody() method.
     * </p>
     */
    private String pathSeparatorStyle;

    /**
     * <p>
     * Represents the configurable separator CSS style that will be used to draw the separator.  This is the actual CSS
     * style that will be rendered to the underlying text. This variable is set by the constructor, is immutable and
     * can be null to specify no specific style (i.e. use the style that is currently in context).
     * </p>
     */
    private final String configPathSeparatorStyle;

    /**
     * <p>
     * Represents the node (non-root and non-current) CSS style that will be used to draw the node's title.  This is
     * the actual CSS style that will be rendered to the underlying link. This variable is set by the constructor, is
     * mutable and can be null to specify no specific style (i.e. use the style that is currently in context). This
     * variable is referenced by the associated getter/setter methods and the doAfterBody() method.
     * </p>
     */
    private String nodeStyle;

    /**
     * <p>
     * Represents the configurable node (non-root and non-current) CSS style that will be used to draw the node's
     * title.  This is the actual CSS style that will be rendered to the underlying link. This variable is set by the
     * constructor, is immutable and can be null to specify no specific style (i.e. use the style that is currently in
     * context).
     * </p>
     */
    private final String configNodeStyle;

    /**
     * <p>
     * Represents the current (only) node CSS style that will be used to draw the node's title.  This is the actual CSS
     * style that will be rendered to the underlying link. This variable is set by the constructor, is mutable and can
     * be null to specify no specific style (i.e. use the style that is currently in context). This variable is
     * referenced by the associated getter/setter methods and the doAfterBody() method.
     * </p>
     */
    private String currentNodeStyle;

    /**
     * <p>
     * Represents the configurable current (only) node CSS style that will be used to draw the node's title.  This is
     * the actual CSS style that will be rendered to the underlying link. This variable is set by the constructor, is
     * immutable and can be null to specify no specific style (i.e. use the style that is currently in context).
     * </p>
     */
    private final String configCurrentNodeStyle;

    /**
     * <p>
     * Represents the root (only) node CSS style that will be used to draw the node's title.  This is the actual CSS
     * style that will be rendered to the underlying link. This variable is set by the constructor, is mutable and can
     * be null to specify no specific style (i.e. use the style that is currently in context). This variable is
     * referenced by the associated getter/setter methods and the doAfterBody() method.
     * </p>
     */
    private String rootNodeStyle;

    /**
     * <p>
     * Represents the configurable root (only) node CSS style that will be used to draw the node's title.  This is the
     * actual CSS style that will be rendered to the underlying link. This variable is set by the constructor, is
     * immutable and can be null to specify no specific style (i.e. use the style that is currently in context).
     * </p>
     */
    private final String configRootNodeStyle;

    /**
     * <p>
     * Represents the mouse over node CSS style that will be used to draw the node's title.  This is the actual CSS
     * style that will be rendered to the underlying link. This variable is set by the constructor, is mutable and can
     * be null to specify no specific style (i.e. use the style that is currently in context). This variable is
     * referenced by the associated getter/setter methods and the doAfterBody() method.
     * </p>
     */
    private String mouseOverStyle;

    /**
     * <p>
     * Represents the configurable mouse over node CSS style that will be used to draw the node's title.  This is the
     * actual CSS style that will be rendered to the underlying link. This variable is set by the constructor, is
     * immutable and can be null to specify no specific style (i.e. use the style that is currently in context).
     * </p>
     */
    private final String configMouseOverStyle;

    /**
     * <p>
     * Represents whether any exceptions should be silently ignored or not. True to silently ignore, false to let
     * exception pass through to JSP engine. This variable is set by the constructor and is mutable. This variable is
     * referenced by the associated getter/setter methods and the doStartTag(), doAfterBody() and doEndTag() methods.
     * </p>
     */
    private boolean silentErrors;

    /**
     * <p>
     * Represents whether any exceptions should be silently ignored or not (specified in the config). True to silently
     * ignore, false to let exception pass through to JSP engine. This variable is set by the constructor and is
     * immutable.
     * </p>
     */
    private final boolean configSilentErrors;

    /**
     * <p>
     * Represents the direction of the path.  The current allowable values are either  ROOT_TO_CURRENT (specifying root
     * [on the left] to current [on the right]) or  CURRENT_TO_ROOT (specifying current [on the left] to root [on the
     * right]). This variable is set by the constructor, is mutable and can only be the above values. This variable is
     * referenced by the associated getter/setter methods and the doStartTag() method.
     * </p>
     */
    private int pathDirection;

    /**
     * <p>
     * Represents the configurable direction of the path.  The current allowable values are either  ROOT_TO_CURRENT
     * (specifying root [on the left] to current [on the right]) or  CURRENT_TO_ROOT (specifying current [on the left]
     * to root [on the right]). This variable is set by the constructor, is immutable and can only be the above
     * values.
     * </p>
     */
    private final int configPathDirection;

    /**
     * <p>
     * Represents the path override for this specific tag. This variable is set initially to null and is mutable.  If
     * null, there is no path override available and normal path discovery should be used.  If non-null, a
     * XmlPathDiscovery class will be constructed with this path and used to return the path specified in this
     * variable. This variable is referenced by the associated getter/setter methods and the doStartTag() method.
     * </p>
     */
    private String pathOverride = null;

    /**
     * <p>
     * Represents the title that will be used if the node is being discovered by this tag (i.e. no other tag has
     * already discovered the node and set the BREADCRUMB_CURRENT_NODE attribute) - otherwise it is ignored. This
     * variable is set initially to null, is mutable and will never be an empty string but can be null.  A null
     * indicates that the title should default to the title specified in the PathDiscovery data source.  This variable
     * is referenced by the  associated getter/setter methods and the doStartTag() method.
     * </p>
     */
    private String title = null;

    /**
     * <p>
     * Represents the iterator over the current node path.  This iterator will initially be null, be mutable by the
     * doStartTag() and will be non-null during the body processing.  The iterator will be created in the doStartTag()
     * from the path discovered by the PathDiscovery implementation. The node formatter (or body if no node formatter
     * specified) will then be called, for each node  in this iterator.
     * </p>
     */
    private ListIterator iterator = null;

    /**
     * <p>
     * The object factory used to create instances of NodeDiscovery and PathDiscovery classes. This will initially be
     * null, and lazily initialized in the doStartTag(). Once initialized, the reference will not change.
     * </p>
     */
    private ObjectFactory objectFactory = null;

    /**
     * <p>
     * The node discovery used to discover the current node based on the page context. This will initially be null, and
     * lazily initialized in the doStartTag(). Once initialized, the reference will not change.
     * </p>
     */
    private NodeDiscovery nodeDiscovery = null;

    /**
     * <p>
     * The path discovery used to discover the current path from the current node to the root node. This will initially
     * be null, and lazily initialized in the doStartTag(). Once initialized, the reference will not change.
     * </p>
     */
    private PathDiscovery pathDiscovery = null;

    /**
     * <p>
     * Constructs the BreadCrumbTrailTag, using the configuration manager to set default values. If a configuration
     * exception occurs, ignore it and use the default values.
     * </p>
     */
    public BreadCrumbTrailTag() {
        super();

        // load the property values from the config
        configPathSeparator = getPropertyValue(PATH_SEPARATOR_PROPERTY, ":", true);
        configPathSeparatorStyle = getPropertyValue(PATH_SEPARATOR_STYLE_PROPERTY, null);
        configNodeStyle = getPropertyValue(NODE_STYLE_PROPERTY, null);
        configCurrentNodeStyle = getPropertyValue(CURRENT_NODE_STYLE_PROPERTY, null);
        configRootNodeStyle = getPropertyValue(ROOT_NODE_STYLE_PROPERTY, null);
        configMouseOverStyle = getPropertyValue(MOUSE_OVER_STYLE_PROPERTY, null);
        configSilentErrors = Boolean.valueOf(getPropertyValue(SILENT_ERRORS_PROPERTY, null)).booleanValue();

        // parse the path direction as an integer
        try {
            pathDirection = Integer.parseInt(getPropertyValue(PATH_DIRECTION_PROPERTY, null));
            if ((pathDirection != ROOT_TO_CURRENT) && (pathDirection != CURRENT_TO_ROOT)) {
                pathDirection = ROOT_TO_CURRENT;
            }
        } catch (NumberFormatException e) {
            pathDirection = ROOT_TO_CURRENT;
        }
        configPathDirection = pathDirection;

        // assign the configured values to the attributes
        resetAttributes();
    }

    /**
     * <p>
     * Returns the value of pathDirection attribute.
     * </p>
     *
     * @return The path direction, either ROOT_TO_CURRENT or CURRENT_TO_ROOT
     */
    public int getPathDirection() {
        return pathDirection;
    }

    /**
     * <p>
     * Sets the value of pathDirection attribute.
     * </p>
     *
     * @param pathDirection The new path direction
     *
     * @throws IllegalArgumentException if pathDirection is not ROOT_TO_CURRENT or CURRENT_TO_ROOT
     */
    public void setPathDirection(int pathDirection) {
        if ((pathDirection != ROOT_TO_CURRENT) && (pathDirection != CURRENT_TO_ROOT)) {
            throw new IllegalArgumentException("pathDirection is not ROOT_TO_CURRENT or CURRENT_TO_ROOT");
        }
        this.pathDirection = pathDirection;
    }

    /**
     * <p>
     * Returns the value of pathSeparator attribute.
     * </p>
     *
     * @return A non-null, possibly empty (trim'd) string representing the path separator
     */
    public String getPathSeparator() {
        return pathSeparator;
    }

    /**
     * <p>
     * Sets the value of pathSeparator attribute.
     * </p>
     *
     * @param pathSeparator A non-null, possibly empty (trim'd) string representing the path separator
     *
     * @throws IllegalArgumentException if the pathSeparator is null
     */
    public void setPathSeparator(String pathSeparator) {
        if (pathSeparator == null) {
            throw new IllegalArgumentException("pathSeparator is null");
        }
        this.pathSeparator = pathSeparator;
    }

    /**
     * <p>
     * Returns the value of pathSeparatorStyle attribute.
     * </p>
     *
     * @return A possibly null, never empty (trim'd) string representing the path separator style
     */
    public String getPathSeparatorStyle() {
        return pathSeparatorStyle;
    }

    /**
     * <p>
     * Sets the value of pathSeparatorStyle attribute.
     * </p>
     *
     * @param pathSeparatorStyle A possibly null, possibly empty (trim'd) string representing the path separator style
     */
    public void setPathSeparatorStyle(String pathSeparatorStyle) {
        this.pathSeparatorStyle = isEmptyString(pathSeparatorStyle) ? null : pathSeparatorStyle;
    }

    /**
     * <p>
     * Returns the value of nodeStyle attribute.
     * </p>
     *
     * @return A possibly null, never empty (trim'd) string representing the node style
     */
    public String getNodeStyle() {
        return nodeStyle;
    }

    /**
     * <p>
     * Sets the value of nodeStyle attribute.
     * </p>
     *
     * @param nodeStyle A possibly null, possibly empty (trim'd) string representing the node style
     */
    public void setNodeStyle(String nodeStyle) {
        this.nodeStyle = isEmptyString(nodeStyle) ? null : nodeStyle;
    }

    /**
     * <p>
     * Returns the value of currentNodeStyle attribute.
     * </p>
     *
     * @return A possibly null, never empty (trim'd) string representing the current node style
     */
    public String getCurrentNodeStyle() {
        return currentNodeStyle;
    }

    /**
     * <p>
     * Sets the value of currentNodeStyle attribute.
     * </p>
     *
     * @param currentNodeStyle A possibly null, possibly empty (trim'd) string representing the current node style
     */
    public void setCurrentNodeStyle(String currentNodeStyle) {
        this.currentNodeStyle = isEmptyString(currentNodeStyle) ? null : currentNodeStyle;
    }

    /**
     * <p>
     * Returns the value of rootNodeStyle attribute.
     * </p>
     *
     * @return A possibly null, never empty (trim'd) string representing the root node style
     */
    public String getRootNodeStyle() {
        return rootNodeStyle;
    }

    /**
     * <p>
     * Sets the value of rootNodeStyle attribute.
     * </p>
     *
     * @param rootNodeStyle A possibly null, possibly empty (trim'd) string representing the root node style
     */
    public void setRootNodeStyle(String rootNodeStyle) {
        this.rootNodeStyle = isEmptyString(rootNodeStyle) ? null : rootNodeStyle;
    }

    /**
     * <p>
     * Returns the value of mouseOverStyle attribute.
     * </p>
     *
     * @return A possibly null, never empty (trim'd) string representing the mouse over style
     */
    public String getMouseOverStyle() {
        return mouseOverStyle;
    }

    /**
     * <p>
     * Sets the value of mouseOverStyle attribute.
     * </p>
     *
     * @param mouseOverStyle A possibly null, possibly empty (trim'd) string representing the mouse over style
     */
    public void setMouseOverStyle(String mouseOverStyle) {
        this.mouseOverStyle = isEmptyString(mouseOverStyle) ? null : mouseOverStyle;
    }

    /**
     * <p>
     * Returns the value of pathOverride attribute.
     * </p>
     *
     * @return A possibly null, never empty (trim'd) string representing the path override
     */
    public String getPathOverride() {
        return pathOverride;
    }

    /**
     * <p>
     * Sets the value of pathOverride attribute.
     * </p>
     *
     * @param pathOverride A possibly null, possibly empty (trim'd) string representing the path override
     */
    public void setPathOverride(String pathOverride) {
        this.pathOverride = isEmptyString(pathOverride) ? null : pathOverride;
    }

    /**
     * <p>
     * Returns the value of silentErrors attribute.
     * </p>
     *
     * @return True if exceptions will be silently ignored, false if they should be thrown to the JSP Engine
     */
    public boolean getSilentErrors() {
        return silentErrors;
    }

    /**
     * <p>
     * Sets the value of silentErrors attribute.
     * </p>
     *
     * @param silentErrors True if exceptions will be silently ignored, false if they should be thrown to the JSP
     *        Engine
     */
    public void setSilentErrors(boolean silentErrors) {
        this.silentErrors = silentErrors;
    }

    /**
     * <p>
     * Returns the value of title attribute.
     * </p>
     *
     * @return A possibly null, never empty (trim'd) string representing the current node title
     */
    public String getTitle() {
        return title;
    }

    /**
     * <p>
     * Sets the value of title attribute.
     * </p>
     *
     * @param title A possibly null, possibly empty (trim'd) string representing the title of the current node
     */
    public void setTitle(String title) {
        this.title = isEmptyString(title) ? null : title;
    }

    /**
     * <p>
     * Prints the style settings to the output. Basically it first discovers the current node, then discovers the
     * current path from the root to the current node, and writes the CSS styles for the nodes in the current path.
     * </p>
     *
     * @return EVAL_BODY_BUFFERED to continue evaluating the body, or SKIP_BODY if the current path is not available,
     *         or any error occurs and silentErrors is true
     *
     * @throws BreadCrumbException if creating of bread crumb trail failed and silentErrors is false
     */
    public int doStartTag() throws BreadCrumbException {
        try {
            // lazily initialize the objects to compose the bread crumb trail
            if (objectFactory == null) {
                objectFactory = new ObjectFactory(new ConfigManagerSpecificationFactory(getClass().getName()));
            }
            if (nodeDiscovery == null) {
                nodeDiscovery = (NodeDiscovery) objectFactory.createObject(NodeDiscovery.class);
            }
            if (pathDiscovery == null) {
                pathDiscovery = (PathDiscovery) objectFactory.createObject(PathDiscovery.class);
            }

            // setup the parameters for node discovery
            if (nodeDiscovery instanceof AbstractNodeDiscovery) {
                ((AbstractNodeDiscovery) nodeDiscovery).setTitle(title);
            }

            // discover the current node - depends on whether the BREADCRUMB_CURRENT_NODE attribute is specified
            Object value = pageContext.getAttribute(BREADCRUMB_CURRENT_NODE);
            BreadCrumbNode currentNode = null;

            if (!(value instanceof BreadCrumbNode)) {
                currentNode = nodeDiscovery.getCurrentNode(pageContext);
                if (currentNode == null) {
                    throw new BreadCrumbException("Unable to discover the current node");
                }
            } else {
                currentNode = (BreadCrumbNode) value;
            }

            // discover the current path - depends on whether the pathOverride attribute is specified
            PathDiscovery pathDiscovery = (pathOverride == null) ? this.pathDiscovery
                                                                 : new XmlPathDiscovery(pathOverride, false);

            BreadCrumbNode[] nodes = pathDiscovery.getPath(currentNode, pageContext);

            if (nodes == null) {
                return SKIP_BODY;
            }

            // prepare the iterator over the nodes in the current path
            // (for rendering by the node formatter)
            List list = Arrays.asList(nodes);

            if (pathDirection == CURRENT_TO_ROOT) {
                Collections.reverse(list);
            }
            iterator = list.listIterator();

            // write the styles to the output if specified
            JspWriter out = pageContext.getOut();

            if (isStyleSpecified()) {
                out.println("<span>");
                printStyleSettings(out);
            }

            // continue with the node formatting in the body
            return EVAL_BODY_BUFFERED;
        } catch (Exception e) {
            processException(e);
            return SKIP_BODY;
        }
    }

    /**
     * <p>
     * Gets the next item from the iterator and sets the attribute information for what needs to be formatted. This
     * method will set the BREADCRUMB_FORMAT_NODE attribute to the next node to be formatted and the
     * BREADCRUMB_FORMAT_STYLE_CLASS attribute to the CSS style class to apply to the node (if the style is
     * specified). These attributes will be used by the node formatter inside the body.
     * </p>
     */
    public void doInitBody() {
        initBody();
    }

    /**
     * <p>
     * Gets the body content text and writes it out for the next node to be formatted. The body content represents the
     * node formatter (possibly using node formatter tag) to render the next node.
     * </p>
     *
     * @return EVAL_BODY_AGAIN to process the next node, or SKIP_BODY if no more node is available, or any error occurs
     *         and silentErrors is true
     *
     * @throws BreadCrumbException if creating of bread crumb trail failed and silentErrors is false
     */
    public int doAfterBody() throws BreadCrumbException {
        try {
            String str = bodyContent.getString();
            JspWriter out = getPreviousOut();

            // write the non-empty body content
            if (!isEmptyString(str)) {
                out.print(str);
            }

            // clear the body content for next iteration
            bodyContent.clearBody();

            if (iterator.hasNext()) {
                // if next node is available, write the separator and continue with the iteration
                printSeparator(out);
                initBody();
                return EVAL_BODY_AGAIN;
            } else {
                // otherwise stop the iteration
                return SKIP_BODY;
            }
        } catch (IOException e) {
            processException(e);
            return SKIP_BODY;
        }
    }

    /**
     * <p>
     * Prints the style settings to the output. Basically it writes the closing span tag.
     * </p>
     *
     * @return Always EVAL_PAGE to continue evaluating the page
     *
     * @throws BreadCrumbException if creating of bread crumb trail failed and silentErrors is false
     */
    public int doEndTag() throws BreadCrumbException {
        try {
            // end the span tag if style is specified
            if (isStyleSpecified()) {
                JspWriter out = (bodyContent == null) ? pageContext.getOut() : getPreviousOut();

                out.println("</span>");
            }
        } catch (IOException e) {
            processException(e);
        }

        // continue evaluating the rest of the page
        return EVAL_PAGE;
    }

    /**
     * <p>
     * Resets all attribute member variables to their initial values.
     * </p>
     */
    public void release() {
        resetAttributes();
        super.release();
    }

    /**
     * <p>
     * Prints the style settings out to the specified writer. This method assumes at least one style is non-null.
     * </p>
     *
     * @param out The non-null JspWriter to write the style settings to
     *
     * @throws IOException if an I/O error occurs writing out the style settings
     */
    private void printStyleSettings(JspWriter out) throws IOException {
        // start the style tag
        out.println("<style>");

        // write the style if it is specified
        if (pathSeparatorStyle != null) {
            printStyle(out, PATH_SEPARATOR_STYLE, pathSeparatorStyle);
        }
        if (nodeStyle != null) {
            printStyle(out, composeStyle(NODE_STYLE, LINK_STYLE), nodeStyle);
        }
        if (currentNodeStyle != null) {
            printStyle(out, composeStyle(CURRENT_NODE_STYLE, LINK_STYLE), currentNodeStyle);
        }
        if (rootNodeStyle != null) {
            printStyle(out, composeStyle(ROOT_NODE_STYLE, LINK_STYLE), rootNodeStyle);
        }
        if (mouseOverStyle != null) {
            printStyle(out, composeStyle(NODE_STYLE, HOVER_STYLE), nodeStyle);
            printStyle(out, composeStyle(CURRENT_NODE_STYLE, HOVER_STYLE), currentNodeStyle);
            printStyle(out, composeStyle(ROOT_NODE_STYLE, HOVER_STYLE), rootNodeStyle);
        }

        // end the style tag
        out.println("</style>");
    }

    /**
     * <p>
     * Prints the specified style out to the specified writer.
     * </p>
     *
     * @param out The non-null JspWriter to write the style to
     * @param style The non-null, non-empty (trim'd) string representing the style to write
     * @param value The non-null, non-empty (trim'd) string representing the value of the style to write
     *
     * @throws IOException if an I/O error occurs writing out the style
     */
    private void printStyle(JspWriter out, String style, String value)
        throws IOException {
        out.print(style);
        out.print(" {");
        out.print(value);
        out.println("}");
    }

    /**
     * <p>
     * Prints the separator with the separator style (if specified) to the specified writer.
     * </p>
     *
     * @param out The non-null JspWriter to write the separator to
     *
     * @throws IOException if an I/O error occurs writing out the separator
     */
    private void printSeparator(JspWriter out) throws IOException {
        if (pathSeparatorStyle != null) {
            // if path separator style is specified, wrap the path separator with span tag
            out.print("<span class=\"");
            out.print(PATH_SEPARATOR_STYLE);
            out.print("\">");
            out.print(pathSeparator);
            out.print("</span>");
        } else {
            // otherwise just write the path separator
            out.print(pathSeparator);
        }
    }

    /**
     * <p>
     * Processes the caught exception for the tag-related methods. If silentErrors is true, the exception is simply
     * ignored. Otherwise the exception will be thrown as a BreadCrumbException.
     * </p>
     *
     * @param e the exception to process.
     *
     * @throws BreadCrumbException if silentErrors is false.
     */
    private void processException(Exception e) throws BreadCrumbException {
        // suppress errors if silentErrors is true
        if (silentErrors) {
            return;
        }

        if (e instanceof BreadCrumbException) {
            // re-throw the BreadCrumbException caught
            throw (BreadCrumbException) e;
        } else {
            // or wrap the exception as BreadCrumbException and throw it
            throw new BreadCrumbException("Fails to process bread crumb trail tag", e);
        }
    }

    /**
     * <p>
     * Composes the style string based on the given style class and sub-style.
     * </p>
     *
     * @param styleClass the CSS style class.
     * @param subStyle the sub-style associated with the style class.
     *
     * @return the style string.
     */
    private static String composeStyle(String styleClass, String subStyle) {
        StringBuffer sb = new StringBuffer();

        sb.append(".");
        sb.append(styleClass);
        sb.append(" ");
        sb.append(subStyle);

        return sb.toString();
    }

    /**
     * <p>
     * Resets all attribute member variables to their initial values. If the attribute is configurable, the initial
     * value is the one loaded from configuration.
     * </p>
     */
    private void resetAttributes() {
        // reset configurable attributes
        pathSeparator = configPathSeparator;
        pathSeparatorStyle = configPathSeparatorStyle;
        nodeStyle = configNodeStyle;
        currentNodeStyle = configCurrentNodeStyle;
        rootNodeStyle = configRootNodeStyle;
        mouseOverStyle = configMouseOverStyle;
        silentErrors = configSilentErrors;
        pathDirection = configPathDirection;

        // reset non-configurable attributes
        pathOverride = null;
        title = null;
    }

    /**
     * <p>
     * Gets the property value of the specified name. If the value is not specified or an empty string, or any
     * configuration exception occurs, the default value will be used.
     * </p>
     *
     * @param name the name of the property.
     * @param defaultValue the default value to use.
     *
     * @return the value of the property.
     */
    private static String getPropertyValue(String name, String defaultValue) {
        return getPropertyValue(name, defaultValue, false);
    }

    /**
     * <p>
     * Gets the property value of the specified name. If the value is not specified or an empty string (when canBeEmpty
     * is false), or any configuration exception occurs, the default value will be used.
     * </p>
     *
     * @param name the name of the property.
     * @param defaultValue the default value to use.
     * @param canBeEmpty whether the property value can be an empty (trim'd) string
     *
     * @return the value of the property.
     */
    private static String getPropertyValue(String name, String defaultValue, boolean canBeEmpty) {
        try {
            String value = ConfigManager.getInstance().getString(NAMESPACE, name);

            // use default value if property is not specified or has empty value
            return ((value == null) || (!canBeEmpty && isEmptyString(value))) ? defaultValue : value;
        } catch (ConfigManagerException e) {
            // use default value if any error occurs
            return defaultValue;
        }
    }

    /**
     * <p>
     * Checks whether the given string is an empty (trim'd) string.
     * </p>
     *
     * @param str the string to test.
     *
     * @return true if it is an empty (trim'd) string, false otherwise.
     */
    private static boolean isEmptyString(String str) {
        return (str != null) && (str.trim().length() == 0);
    }

    /**
     * <p>
     * Checks if any style is specified for the bread crumb trail. If it is true, then the span tag should be rendered.
     * </p>
     *
     * @return true if any style is specified, false otherwise.
     */
    private boolean isStyleSpecified() {
        return (pathSeparatorStyle != null) || (nodeStyle != null) || (currentNodeStyle != null)
        || (rootNodeStyle != null) || (mouseOverStyle != null);
    }

    /**
     * <p>
     * Gets the next item from the iterator and sets the attribute information for what needs to be formatted. This
     * method will set the BREADCRUMB_FORMAT_NODE attribute to the next node to be formatted and the
     * BREADCRUMB_FORMAT_STYLE_CLASS attribute to the CSS style class to apply to the node (if the style is
     * specified). These attributes will be used by the node formatter inside the body.
     * </p>
     *
     * <p>
     * This method must be called before each iteration of the body.
     * </p>
     */
    private void initBody() {
        // save the next node to be formatted
        pageContext.setAttribute(BREADCRUMB_FORMAT_NODE, iterator.next());

        // determine the style class for the next node
        String styleClass = null;
        String style = null;

        // preference order is root over current over node
        if (((iterator.previousIndex() == 0) && (pathDirection == ROOT_TO_CURRENT))
                || (!iterator.hasNext() && (pathDirection == CURRENT_TO_ROOT))) {
            styleClass = ROOT_NODE_STYLE;
            style = rootNodeStyle;
        } else if ((!iterator.hasNext() && (pathDirection == ROOT_TO_CURRENT))
                || ((iterator.previousIndex() == 0) && (pathDirection == CURRENT_TO_ROOT))) {
            styleClass = CURRENT_NODE_STYLE;
            style = currentNodeStyle;
        } else {
            styleClass = NODE_STYLE;
            style = nodeStyle;
        }

        // save the CSS style class if the style is specified
        if ((mouseOverStyle != null) || (style != null)) {
            pageContext.setAttribute(BREADCRUMB_FORMAT_STYLE_CLASS, styleClass);
        }
    }
}
