/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;

import java.io.IOException;

import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.TagSupport;


/**
 * <p>
 * This empty tag will format the node and title using the html anchor pattern of <code>&lt;a
 * href="url"&gt;title&lt;/a&gt;</code>. This tag has no attributes.
 * </p>
 *
 * <p>
 * <b>Implementation notes:</b> This node formatter tag will be used in the body of a BreadCrumbTrailTag and will
 * format the current node and title using an html anchor.
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> This class is thread safe in the context of the JSP rendering process.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public class HtmlAnchorNodeFormatterTag extends TagSupport {
    /**
     * <p>
     * The title string to use in the html anchor if the title is not available from the node.
     * </p>
     */
    private static final String DEFAULT_TITLE = "Unspecified";

    /**
     * <p>
     * Constructs the HtmlAnchorNodeFormatterTag. Simply calls the superclass constructor.
     * </p>
     */
    public HtmlAnchorNodeFormatterTag() {
        super();
    }

    /**
     * <p>
     * This method will format the current node and style as a simple html anchor and then write the results to the
     * output.
     * </p>
     *
     * <p>
     * This method assumes that:
     *
     * <ul>
     * <li>
     * the BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE attribute holds the current node.
     * </li>
     * <li>
     * the BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS attribute holds the CSS style class for the current node if
     * it is available.
     * </li>
     * </ul>
     *
     * If any of these is not met, a BreadCrumbException will be thrown.
     * </p>
     *
     * @return Always SKIP_BODY to skip the empty body
     *
     * @throws BreadCrumbException if some exception occurs during processing
     */
    public int doStartTag() throws BreadCrumbException {
        // get the format parameters from the attributes
        BreadCrumbNode node = FormatterUtil.getFormatNode(pageContext);
        String styleClass = FormatterUtil.getFormatStyleClass(pageContext);

        try {
            JspWriter out = pageContext.getOut();

            // open the html anchor tag
            out.print("<a href=\"");
            out.print(node.getUrl());
            out.print("\"");

            // write the style class attribute if available
            if (styleClass != null) {
                out.print(" class=\"");
                out.print(styleClass);
                out.print("\"");
            }
            out.print(">");

            // write the title of the link
            if (node.getTitle() != null) {
                out.print(node.getTitle());
            } else {
                out.print(DEFAULT_TITLE);
            }

            // close the html anchor tag
            out.print("</a>");

            return SKIP_BODY;
        } catch (IOException e) {
            throw new BreadCrumbException("Some I/O error occurs", e);
        }
    }
}
