/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

/**
 * <p>
 * This abstract implementation of the NodeMatcher interface provides title and url type services to subclasses.
 * </p>
 *
 * <p>
 * <b>Implementation notes:</b> The developer should strongly encourage that all implementations of the NodeMatcher
 * inherit from this abstract base.  This abstract base currently only provides title/url services but may provide
 * additional services in the future.
 * </p>
 *
 * <p>
 * <b>Thread safety:</b> This class is thread safe by being immutable.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public abstract class AbstractNodeMatcher implements NodeMatcher {
    /**
     * <p>
     * Represents the default title that a node can take if it matches this node matcher.  This variable is set  by the
     * constructor, is immutable and can never be null nor an empty (trim'd) string.  The title will be referenced in
     * the getTitle() method.
     * </p>
     */
    private final String title;

    /**
     * <p>
     * Represents the default url that can be used if no contextual information can be used.  This variable is set  by
     * the constructor, is immutable and can never be null nor an empty (trim'd) string.  The url will be referenced
     * in the getUrl() method.
     * </p>
     */
    private final String url;

    /**
     * <p>
     * Constructs the abstract base from the given title and url.
     * </p>
     *
     * @param title A non-null, non-empty (trim'd) title to use
     * @param url A non-null, non-empty (trim'd) url to use
     *
     * @throws IllegalArgumentException if any argument is null or an empty (trim'd) string
     */
    protected AbstractNodeMatcher(String title, String url) {
        if (title == null) {
            throw new IllegalArgumentException("title is null");
        }
        if (title.trim().length() == 0) {
            throw new IllegalArgumentException("title is empty string");
        }
        if (url == null) {
            throw new IllegalArgumentException("url is null");
        }
        if (url.trim().length() == 0) {
            throw new IllegalArgumentException("url is empty string");
        }
        this.title = title;
        this.url = url;
    }

    /**
     * <p>
     * Returns the default title that the nodes that match this node matcher can use if their title is not specified.
     * </p>
     *
     * @return A non-null, non-empty (trim'd) string representing the default title
     */
    public String getTitle() {
        return title;
    }

    /**
     * <p>
     * Returns the default url that can be used if contextual node information is not available.
     * </p>
     *
     * @return A non-null, non-empty (trim'd) string representing the default url
     */
    public String getUrl() {
        return url;
    }
}
