/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;

import java.io.Serializable;


/**
 * <p>
 * Defines the contract for a node matcher. Implementations need to provide three functions:
 * </p>
 *
 * <ol>
 * <li>
 * A matching function that will take a node and determine if it matches
 * </li>
 * <li>
 * A default title string used for those nodes that match
 * </li>
 * <li>
 * A default url to use if no contextual information is available
 * </li>
 * </ol>
 *
 * <p>
 * <b>Implementation notes:</b> NodeMatcher implementations will be created by SiteDiscovery implementations when the
 * site map is created. The NodeMatcher implementation will then be used by the DynamicPathDiscovery when it attempts
 * to discover the path to the root node from some current node. Implementations of this inteface
 * <strong>must</strong> override the equals() and hashCode() methods because the implementation instance will be
 * stored in a map.
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> Implementations of this interface should be thread safe.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public interface NodeMatcher extends Serializable {
    /**
     * <p>
     * Determines whether the specified node matches this node matcher.
     * </p>
     *
     * @param node A non-null node to test
     *
     * @return true if the node matches, false otherwise
     *
     * @throws IllegalArgumentException if the node is null
     */
    boolean matches(BreadCrumbNode node);

    /**
     * <p>
     * Returns the default title that the nodes that match this node matcher can use if their title is not specified.
     * </p>
     *
     * @return A non-null, non-empty (trim'd) string representing the default title
     */
    String getTitle();

    /**
     * <p>
     * Returns the default url that can be used if contextual node information is not available.
     * </p>
     *
     * @return A non-null, non-empty (trim'd) string representing the default url
     */
    String getUrl();
}
