/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;

import java.io.Serializable;


/**
 * <p>
 * This class is a typical pair class that holds a paired association of a node matcher to the matching node. The node
 * matcher describes the default node information. The node describes the contextual specific information.
 * </p>
 *
 * <p>
 * <b>Implementation notes:</b> This class will be created by the DynamicPathDiscovery class as it's creating the
 * current path. The DynamicPathDiscovery will then use this information to create the path from each of the node
 * pairs. This class provides one additional benefit - the returned BreadCrumbNode will contain either the specified
 * title (in the node itself) or if the title is null, the title from the NodeMatcher instead.
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> This class is thread safe by being immutable.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
class NodePair implements Serializable {
    /**
     * <p>
     * Represents the actual node that holds the contextual information (query string, etc) that matches the node
     * matcher.  This variable is set  by the constructor, is immutable and can never be null.  The node will be
     * referenced in the getNode() method.
     * </p>
     */
    private final BreadCrumbNode node;

    /**
     * <p>
     * Represents the node matcher that holds the matching information and the default title and url.  This variable is
     * set  by the constructor, is immutable and can be null (if there is no associated nodeMatcher).  The  node
     * matcher will be referenced in the getNodeMatcher() method.
     * </p>
     */
    private final NodeMatcher nodeMatcher;

    /**
     * <p>
     * Constructs the node pair from the given node and node matcher.
     * </p>
     *
     * @param node The non-null bread crumb node that matches
     * @param nodeMatcher The possibly null node matcher (null if there is no associated nodeMatcher)
     *
     * @throws IllegalArgumentException if node is null
     */
    NodePair(BreadCrumbNode node, NodeMatcher nodeMatcher) {
        if (node == null) {
            throw new IllegalArgumentException("node is null");
        }

        // if title is not specified in the node, use the one from node matcher
        this.node = ((node.getTitle() == null) && (nodeMatcher != null))
            ? new BreadCrumbNode(node.getUrl(), nodeMatcher.getTitle()) : node;
        this.nodeMatcher = nodeMatcher;
    }

    /**
     * <p>
     * A getter method for the node. If the title of the node given in the constructor is null, the returned node will
     * have the title from the node matcher if there is one.
     * </p>
     *
     * @return A non-null node that matches
     */
    public BreadCrumbNode getNode() {
        return node;
    }

    /**
     * <p>
     * The getter method for the node matcher.
     * </p>
     *
     * @return A possibly null node matcher (null if there is no associated nodeMatcher)
     */
    public NodeMatcher getNodeMatcher() {
        return nodeMatcher;
    }
}
