/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.jsp.PageContext;


/**
 * A class to find the shortest paths from the current node to a root based on a sitemap. Cyclic paths can be detected.
 * This class is package-private so that it can be used by classes in the same package.
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
class ShortestPathFinder {
    /**
     * A map of NodeMatcher keys with a List of NodeMatcher elements as the value. The key NodeMatcher represents the
     * child and the value List represents the parent values. The value can be null to represent no parent elements.
     * This variable is set by the constructor, is immutable and can never be null.
     */
    private final Map reverseSiteMap;

    /**
     * A map of NodeMatcher keys with a List of List of NodeMatcher elements as the value. The key NodeMatcher
     * represents the node to find the shortest paths for, the value represents the shortest paths (can be more than
     * one) from the root to the node. The value can be null to represent no shortest path is available (e.g. because
     * of cyclic path). This variable is initialized by the constructor as an empty map, but the elements are added
     * and cached (for all encountered nodes) when the findShortestPaths() method is called. The results are cached
     * because the shortest paths should be fixed for every node.
     */
    private final Map shortestPaths;

    /**
     * The best depth for the shortest paths found for the target node. It is used to compare with the current depth to
     * avoid the traversal from going too deep. A value of 0 means the best depth has not been found. This variable is
     * used and updated in the buildShortestPaths() method.
     */
    private int bestDepth = 0;

    /**
     * Creates a new instance of ShortestPathFinder based on the given reverse sitemap.
     *
     * @param reverseSiteMap A non-null map of child NodeMatcher to a List of NodeMatcher parents
     */
    public ShortestPathFinder(Map reverseSiteMap) {
        this.reverseSiteMap = reverseSiteMap;
        shortestPaths = new HashMap();
    }

    /**
     * Finds the shortest paths (can be more than one) for the given BreadCrumbNode. This method first selects the node
     * matchers that match the BreadCrumbNode, then builds the shortest paths upon these node matchers.
     *
     * @param node the current node to build the shortest paths for.
     *
     * @return the shortest paths where each path starts from the root node to the child node.
     */
    public NodeMatcher[][] findShortestPaths(BreadCrumbNode node) {
        List childPaths = new ArrayList();
        int childDepth = 0;

        for (Iterator i = reverseSiteMap.keySet().iterator(); i.hasNext();) {
            NodeMatcher child = (NodeMatcher) i.next();

            // test the node against each child node matcher
            if (!child.matches(node)) {
                continue;
            }

            // build the shortest paths for the matched node
            buildShortestPaths(child, new LinkedList(), 0);

            List paths = (List) shortestPaths.get(child);

            // if no shortest path is found, try another node
            if (paths == null) {
                continue;
            }

            int depth = ((List) paths.get(0)).size();

            if ((childDepth == 0) || (depth <= childDepth)) {
                // if a shorter path is found, clear all previous results
                if (depth < childDepth) {
                    childPaths.clear();
                }

                // add the shortest paths from the matched node
                childPaths.addAll(paths);
                childDepth = depth;
            }
        }

        // if no shortest path is found for all matched nodes, return null
        if (childPaths.isEmpty()) {
            return null;
        }

        // convert the list of list to a 2D array
        NodeMatcher[][] nodeMatchers = new NodeMatcher[childPaths.size()][childDepth];

        for (int i = 0; i < nodeMatchers.length; i++) {
            List childPath = (List) childPaths.get(i);

            for (int j = 0; j < nodeMatchers[i].length; j++) {
                nodeMatchers[i][j] = (NodeMatcher) childPath.get(j);
            }
        }
        return nodeMatchers;
    }

    /**
     * Gets the site map from the attribute of the given page context. If not available, the site map is built from the
     * site discovery, and is saved to the attribute in the session scope.
     *
     * @param pageContext the page context to use.
     * @param siteMapAttr the name of the attribute referencing the site map.
     * @param siteDiscovery the site discovery to build the site map if necessary.
     *
     * @return the site map.
     *
     * @throws BreadCrumbException if an exception occurs getting the site map.
     */
    public static Map getSiteMap(PageContext pageContext, String siteMapAttr, SiteDiscovery siteDiscovery)
        throws BreadCrumbException {
        Object obj = pageContext.getAttribute(siteMapAttr, PageContext.SESSION_SCOPE);
        Map siteMap = null;

        // if not available, save the site map to the attribute
        if (!(obj instanceof Map)) {
            siteMap = siteDiscovery.getSiteMap();
            pageContext.setAttribute(siteMapAttr, siteMap, PageContext.SESSION_SCOPE);
        } else {
            siteMap = (Map) obj;
        }
        return siteMap;
    }

    /**
     * Gets the reverse site map from the attribute of the given page context. If not available, the reverse site map
     * is built from the original site map, and both are saved to the attributes in the session scope.
     *
     * @param pageContext the page context to use.
     * @param siteMapAttr the name of the attribute referencing the original site map.
     * @param reverseSiteMapAttr the name of the attribute referencing the reverse site map.
     * @param siteDiscovery the site discovery to build the site map if necessary.
     *
     * @return the reverse site map.
     *
     * @throws BreadCrumbException if an exception occurs getting the reverse site map.
     */
    public static Map getReverseSiteMap(PageContext pageContext, String siteMapAttr, String reverseSiteMapAttr,
        SiteDiscovery siteDiscovery) throws BreadCrumbException {
        Object obj = pageContext.getAttribute(reverseSiteMapAttr, PageContext.SESSION_SCOPE);
        Map reverseSiteMap = null;

        // if not available, save the site map and reverse site map to the attributes
        if (!(obj instanceof Map)) {
            // make sure the original site map is available
            Map siteMap = getSiteMap(pageContext, siteMapAttr, siteDiscovery);

            // build the reverse site map from the original site map - this is done only once
            reverseSiteMap = ShortestPathFinder.buildReverseSiteMap(siteMap);
            pageContext.setAttribute(reverseSiteMapAttr, reverseSiteMap, PageContext.SESSION_SCOPE);
        } else {
            reverseSiteMap = (Map) obj;
        }
        return reverseSiteMap;
    }

    /**
     * Builds the reverse site map (from child to parents) from the original site map (from parent to children). This
     * is done for faster searching of the root node.
     *
     * @param siteMap A non-null map of parent NodeMatcher to a List of NodeMatcher children
     *
     * @return A non-null map of child NodeMatcher to a List of NodeMatcher parents
     */
    private static Map buildReverseSiteMap(Map siteMap) {
        Map reverseSiteMap = new HashMap();

        for (Iterator i = siteMap.entrySet().iterator(); i.hasNext();) {
            Map.Entry entry = (Map.Entry) i.next();
            NodeMatcher parent = (NodeMatcher) entry.getKey();
            List children = (List) entry.getValue();

            // check if there is any child
            if (children == null) {
                continue;
            }

            for (Iterator j = children.iterator(); j.hasNext();) {
                NodeMatcher child = (NodeMatcher) j.next();
                List parents = (List) reverseSiteMap.get(child);

                // if the child does not appear before, add an entry to the map
                if (parents == null) {
                    parents = new ArrayList();
                    reverseSiteMap.put(child, parents);
                }

                // add the parent to the list of parents
                parents.add(parent);
            }
        }
        return reverseSiteMap;
    }

    /**
     * Builds the shortest paths for the given child, and all the parent nodes above the child. The results are cached
     * because the shortest paths should be fixed for every node. For efficiency, this method will not go further the
     * parent nodes if it knows a better path is already found. This is especially useful when the searching of
     * shortest paths is done for several matched child node matchers.
     *
     * @param child the child node to build the shortest paths for.
     * @param visited a list of visited nodes (NodeMatcher), for detecting cyclic path.
     * @param currentDepth the current depth reached so far, should be 0 for the target node.
     */
    private void buildShortestPaths(NodeMatcher child, List visited, int currentDepth) {
        // check if a better path has been reached
        if (currentDepth == 0) {
            bestDepth = 0;
        } else if ((bestDepth > 0) && (currentDepth >= bestDepth)) {
            return;
        }

        // check if the shortest paths have been found
        if (shortestPaths.containsKey(child)) {
            return;
        }

        // check for cyclic path
        if (visited.contains(child)) {
            return;
        }

        List parents = (List) reverseSiteMap.get(child);
        List childPaths = new ArrayList();
        int childDepth = 0;

        // check if the child is already a root
        if (parents == null) {
            childPaths.add(Collections.singletonList(child));
            shortestPaths.put(child, childPaths);
            return;
        }

        // mark this child as visited
        visited.add(child);

        for (Iterator i = parents.iterator(); i.hasNext();) {
            NodeMatcher parent = (NodeMatcher) i.next();

            // traverse the path from each of its parents
            buildShortestPaths(parent, visited, currentDepth + 1);

            List parentPaths = (List) shortestPaths.get(parent);

            // if the parent forms the cyclic path, or goes too deep, try another one
            if (parentPaths == null) {
                continue;
            }

            int parentDepth = ((List) parentPaths.get(0)).size();

            if ((childDepth == 0) || ((parentDepth + 1) <= childDepth)) {
                // if a shorter path is found, clear all previous results
                if ((parentDepth + 1) < childDepth) {
                    childPaths.clear();
                }

                // add all the shortest paths from the parent, and append the child to each
                for (Iterator j = parentPaths.iterator(); j.hasNext();) {
                    List childPath = new ArrayList((List) j.next());

                    childPath.add(child);
                    childPaths.add(childPath);
                }
                childDepth = parentDepth + 1;

                // update the best depth for the target node
                if (currentDepth == 0) {
                    bestDepth = childDepth;
                }
            }
        }

        // add the shortest paths to the map
        if (childPaths.isEmpty()) {
            childPaths = null;
        }
        shortestPaths.put(child, childPaths);

        // mark the child as unvisited, so that its children can attempt another path
        visited.remove(child);
    }
}
