/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.ConfigHelper;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers.RegexNodeMatcher;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.cactus.JspTestCase;


/**
 * Unit tests for DynamicPathDiscovery implementation.
 *
 * <p>
 * Here we assume the handling of cyclic path and the building of site map are tested in the XmlPathDiscoveryTest. They
 * are not repeated here. Rather the tests here focus on building the current path dynamically.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class DynamicPathDiscoveryTest extends JspTestCase {
    /**
     * The DynamicPathDiscovery instance to test against.
     */
    private DynamicPathDiscovery pathDiscovery = null;

    /**
     * The current node to get the path with.
     */
    private BreadCrumbNode node = null;

    /**
     * The current path to learn dynamically from.
     */
    private NodePair[] currentPath = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(DynamicPathDiscoveryTest.class);

        return suite;
    }

    /**
     * Prepares a DynamicPathDiscovery instance for testing. The associated site map has this structure: a->b->d,
     * a->c->e. The current path is set as a->b->c. The current node does not match the site map in the beginning.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    protected void setUp() throws Exception {
        pathDiscovery = new DynamicPathDiscovery();
        node = new BreadCrumbNode("f", "title");

        // make the current path
        String[] urls = {"a", "b", "c"};

        currentPath = new NodePair[urls.length];
        for (int i = 0; i < currentPath.length; i++) {
            currentPath[i] = new NodePair(new BreadCrumbNode(urls[i], "title"),
                    new RegexNodeMatcher(urls[i], urls[i], urls[i]));
        }

        // set the current path to the attribute
        pageContext.setAttribute(DynamicPathDiscovery.CURRENT_PATH, currentPath, pageContext.SESSION_SCOPE);

        ConfigHelper.loadConfig();
    }

    /**
     * Clears all the namespaces.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    protected void tearDown() throws Exception {
        ConfigHelper.unloadConfig();
    }

    /**
     * Tests accessibility of constructor.
     */
    public void testConstructor() {
        new DynamicPathDiscovery();
    }

    /**
     * Test of getPath method with null current node. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_NullCurrentNode() throws Exception {
        try {
            pathDiscovery.getPath(null, pageContext);
            fail("Gets path with null current node");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getPath method with null page context. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_NullPageContext() throws Exception {
        try {
            pathDiscovery.getPath(node, null);
            fail("Gets path with null page context");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getPath method on the site map attribute. Verifies if the site map is stored in the CURRENT_SITEMAP
     * attribute in session scope after calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_OnSiteMapAttr() throws Exception {
        pathDiscovery.getPath(node, pageContext);
        assertNotNull("Fails to set the CURRENT_SITEMAP attribute",
            pageContext.getAttribute(DynamicPathDiscovery.CURRENT_SITEMAP, pageContext.SESSION_SCOPE));
    }

    /**
     * Test of getPath method on the reverse site map attribute. Verifies if the reverse site map is stored in the
     * CURRENT_REVERSE_SITEMAP attribute in session scope after calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_OnReverseSiteMapAttr() throws Exception {
        pathDiscovery.getPath(node, pageContext);
        assertNotNull("Fails to set the CURRENT_REVERSE_SITEMAP attribute",
            pageContext.getAttribute(DynamicPathDiscovery.CURRENT_REVERSE_SITEMAP, pageContext.SESSION_SCOPE));
    }

    /**
     * Test of getPath method on checking the trail backwards. Tests against a current node such that it matches b in
     * the current path a->b->c. Verifies the old elements a are copied, and the current node appended at the end.
     * Also verifies if the new current path is stored in the CURRENT_PATH attribute in session scope after calling
     * it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_OnBackward() throws Exception {
        node = new BreadCrumbNode("b", "title");

        BreadCrumbNode[] path = pathDiscovery.getPath(node, pageContext);

        assertEquals("Returns an incorrect path", 2, path.length);
        assertEquals("Fails to add old elements", currentPath[0].getNode(), path[0]);
        assertEquals("Fails to add current node", node, path[1]);

        currentPath = (NodePair[]) pageContext.getAttribute(DynamicPathDiscovery.CURRENT_PATH, pageContext.SESSION_SCOPE);

        assertNotNull("Fails to set the CURRENT_PATH attribute", currentPath);
        assertEquals("Fails to set the CURRENT_PATH attribute", 2, currentPath.length);
    }

    /**
     * Test of getPath method on checking the trail forwards. Tests against a current node such that it matches the
     * child of c in the current path a->b->c. Verifies the old elements a->b->c are copied, and the current node
     * appended at the end. Also verifies if the new current path is stored in the CURRENT_PATH attribute in session
     * scope after calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_OnForward() throws Exception {
        node = new BreadCrumbNode("e", "title");

        BreadCrumbNode[] path = pathDiscovery.getPath(node, pageContext);

        assertEquals("Returns an incorrect path", 4, path.length);
        for (int i = 0; i < currentPath.length; i++) {
            assertEquals("Fails to add old elements", currentPath[i].getNode(), path[i]);
        }
        assertEquals("Fails to add current node", node, path[currentPath.length]);

        currentPath = (NodePair[]) pageContext.getAttribute(DynamicPathDiscovery.CURRENT_PATH, pageContext.SESSION_SCOPE);

        assertNotNull("Fails to set the CURRENT_PATH attribute", currentPath);
        assertEquals("Fails to set the CURRENT_PATH attribute", 4, currentPath.length);
    }

    /**
     * Test of getPath method on no shortest path found and no current path specified. Tests against a non-matched
     * current node. Should return null. Also verifies if the CURRENT_PATH attribute is still not set.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_NoShortestPath_NoCurrentPath()
        throws Exception {
        // make sure the current path is not set
        pageContext.removeAttribute(DynamicPathDiscovery.CURRENT_PATH, pageContext.SESSION_SCOPE);

        assertNull("Returns an incorrect path", pathDiscovery.getPath(node, pageContext));
        assertNull("Fails to set the CURRENT_PATH attribute",
            pageContext.getAttribute(DynamicPathDiscovery.CURRENT_PATH, pageContext.SESSION_SCOPE));
    }

    /**
     * Test of getPath method on no shortest path found and current path specified. Tests against a non-matched current
     * node. Verifies if the old elements a->b->c are copied, and the current node appended at the end. Also verifies
     * if the new current path is stored in the CURRENT_PATH attribute in session scope after calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_NoShortestPath_CurrentPath()
        throws Exception {
        BreadCrumbNode[] path = pathDiscovery.getPath(node, pageContext);

        assertEquals("Returns an incorrect path", 4, path.length);
        for (int i = 0; i < currentPath.length; i++) {
            assertEquals("Fails to add old elements", currentPath[i].getNode(), path[i]);
        }
        assertEquals("Fails to add current node", node, path[currentPath.length]);

        currentPath = (NodePair[]) pageContext.getAttribute(DynamicPathDiscovery.CURRENT_PATH, pageContext.SESSION_SCOPE);

        assertNotNull("Fails to set the CURRENT_PATH attribute", currentPath);
        assertEquals("Fails to set the CURRENT_PATH attribute", 4, currentPath.length);
    }

    /**
     * Test of getPath method on shortest path found and no current path specified. Tests against a current node
     * matching b in the site map. Verifies if the elements a->b are copied from the shortest path. Also verifies if
     * the new current path is stored in the CURRENT_PATH attribute in session scope after calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_ShortestPath_NoCurrentPath()
        throws Exception {
        // make sure the current node matches
        node = new BreadCrumbNode("b", "title");

        // make sure the current path is not set
        pageContext.removeAttribute(DynamicPathDiscovery.CURRENT_PATH, pageContext.SESSION_SCOPE);

        BreadCrumbNode[] path = pathDiscovery.getPath(node, pageContext);
        String[] expected = {"a", "b"};

        assertEquals("Returns an incorrect path", 2, path.length);
        for (int i = 0; i < path.length; i++) {
            assertEquals("Fails to add elements from shortest path", expected[i], path[i].getUrl());
        }

        currentPath = (NodePair[]) pageContext.getAttribute(DynamicPathDiscovery.CURRENT_PATH, pageContext.SESSION_SCOPE);

        assertNotNull("Fails to set the CURRENT_PATH attribute", currentPath);
        assertEquals("Fails to set the CURRENT_PATH attribute", 2, currentPath.length);
    }

    /**
     * Test of getPath method on shortest path found and current path specified. Tests against a current node matching
     * d in the site map. Verifies if the elements a->b are copied from the current path, and d is appended from the
     * shortest path. Also verifies if the new current path is stored in the CURRENT_PATH attribute in session scope
     * after calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_ShortestPath_CurrentPath()
        throws Exception {
        // make sure the current node matches
        node = new BreadCrumbNode("d", "title");

        BreadCrumbNode[] path = pathDiscovery.getPath(node, pageContext);

        assertEquals("Returns an incorrect path", 3, path.length);
        for (int i = 0; i < 2; i++) {
            assertEquals("Fails to add old elements", currentPath[i].getNode(), path[i]);
        }
        assertEquals("Fails to add elements from shortest path", "d", path[2].getUrl());
    }
}
