/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers.RegexNodeMatcher;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.cactus.JspTestCase;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Unit tests for XmlPathDiscovery implementation.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class XmlPathDiscoveryTest extends JspTestCase {
    /**
     * The XmlPathDiscovery instance to test against.
     */
    private XmlPathDiscovery pathDiscovery = null;

    /**
     * The current node to get the path with.
     */
    private BreadCrumbNode node = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(XmlPathDiscoveryTest.class);

        return suite;
    }

    /**
     * Prepares an XmlPathDiscovery instance for testing. The site map is simply set as a->b. Also prepares a current
     * node that has the url d for the getPath() method.
     */
    protected void setUp() {
        pathDiscovery = new XmlPathDiscovery("<BreadCrumb><bc_node title=\"a\" url=\"/\" pattern=\"a\">"
                + "<bc_node title=\"b\" url=\"/\" pattern=\"b\"/></bc_node></BreadCrumb>", false);
        node = new BreadCrumbNode("d", "title");
    }

    /**
     * Test of constructor with null filename. Expects IllegalArgumentException.
     */
    public void testConstructor_NullFilename() {
        try {
            new XmlPathDiscovery(null);
            fail("Creates XmlPathDiscovery with null filename");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty filename. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyFilename() {
        try {
            new XmlPathDiscovery("");
            fail("Creates XmlPathDiscovery with empty filename");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null source. Expects IllegalArgumentException.
     */
    public void testConstructor_NullSource() {
        try {
            new XmlPathDiscovery(null, false);
            fail("Creates XmlPathDiscovery with null source");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty source. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptySource() {
        try {
            new XmlPathDiscovery("", false);
            fail("Creates XmlPathDiscovery with empty source");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getSiteMap method with invalid source, such as a non-existing file. Expects BreadCrumbException.
     */
    public void testGetSiteMap_InalidSource() {
        pathDiscovery = new XmlPathDiscovery("non-exist");

        try {
            pathDiscovery.getSiteMap();
            fail("Gets site map with invalid source");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * Test of getSiteMap method with valid source. Uses the xml document as in the component specification. Verifies
     * if it returns a map of parent NodeMatcher to a List of NodeMatcher children.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetSiteMap_ValidSource() throws Exception {
        // prepare the xml string source
        String source = "<BreadCrumb><bc_node title=\"MyMain\" url=\"/\" pattern=\"/\">"
            + "<bc_node title=\"Forums\" url=\"/forums\" pattern=\"/forums\">"
            + "<bc_node title=\"Help\" url=\"/forums\" pattern=\"/forums?id.*\"/>"
            + "</bc_node><bc_node title=\"Topics\" url=\"/topics\" pattern=\"/topics\"/>"
            + "</bc_node><bc_node title=\"AltMain\" url=\"/main\" pattern=\"/\">"
            + "<bc_node title=\"Help\" url=\"/forums\" pattern=\"/forums?id.*\"/></bc_node></BreadCrumb>";

        pathDiscovery = new XmlPathDiscovery(source, false);

        // prepare the expected site map
        NodeMatcher myMain = new RegexNodeMatcher("MyMain", "/", "/");
        NodeMatcher forums = new RegexNodeMatcher("Forums", "/forums", "/forums");
        NodeMatcher help = new RegexNodeMatcher("Help", "/forums", "/forums?id.*");
        NodeMatcher topics = new RegexNodeMatcher("Topics", "/topics", "/topics");
        NodeMatcher altMain = new RegexNodeMatcher("AltMain", "/main", "/");

        Map expected = new HashMap();
        List children = new ArrayList();

        // add the relationship for MyMain
        children.add(forums);
        children.add(topics);
        expected.put(myMain, new ArrayList(children));

        // add the relationship for Forums
        children.clear();
        children.add(help);
        expected.put(forums, new ArrayList(children));

        // add the relationship for AltMain
        children.clear();
        children.add(help);
        expected.put(altMain, new ArrayList(children));

        // add the relationship for Topics and Help
        children.clear();
        expected.put(topics, children);
        expected.put(help, children);

        assertEquals("Returns an incorrect site map", expected, pathDiscovery.getSiteMap());
    }

    /**
     * Test of getPath method with null current node. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_NullCurrentNode() throws Exception {
        try {
            pathDiscovery.getPath(null, pageContext);
            fail("Gets path with null current node");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getPath method with null page context. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_NullPageContext() throws Exception {
        try {
            pathDiscovery.getPath(node, null);
            fail("Gets path with null page context");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getPath method on the site map attribute. Verifies if the site map is stored in the CURRENT_SITEMAP
     * attribute in session scope after calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_OnSiteMapAttr() throws Exception {
        // prepare the expected site map
        Map siteMap = new HashMap();
        List children = new ArrayList();

        NodeMatcher a = new RegexNodeMatcher("a", "/", "a");
        NodeMatcher b = new RegexNodeMatcher("b", "/", "b");

        // add the relationship for a
        children.add(b);
        siteMap.put(a, new ArrayList(children));

        // add the relationship for b
        children.clear();
        siteMap.put(b, children);

        pathDiscovery.getPath(node, pageContext);
        assertEquals("Fails to set the CURRENT_SITEMAP attribute", siteMap,
            pageContext.getAttribute(XmlPathDiscovery.CURRENT_SITEMAP, pageContext.SESSION_SCOPE));
    }

    /**
     * Test of getPath method on the reverse site map attribute. Verifies if the reverse site map is stored in the
     * CURRENT_REVERSE_SITEMAP attribute in session scope after calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_OnReverseSiteMapAttr() throws Exception {
        // prepare the expected reverse site map
        Map reverseSiteMap = new HashMap();
        List children = new ArrayList();

        children.add(new RegexNodeMatcher("a", "/", "a"));
        reverseSiteMap.put(new RegexNodeMatcher("b", "/", "b"), children);

        pathDiscovery.getPath(node, pageContext);
        assertEquals("Fails to set the CURRENT_REVERSE_SITEMAP attribute", reverseSiteMap,
            pageContext.getAttribute(XmlPathDiscovery.CURRENT_REVERSE_SITEMAP, pageContext.SESSION_SCOPE));
    }

    /**
     * Test of getPath method with non-matched current node. Should return null.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_NonMatchedCurrentNode() throws Exception {
        assertNull("Returns an incorrect path", pathDiscovery.getPath(node, pageContext));
    }

    /**
     * Test of getPath method with cyclic path in the site map. Composes the site map as cyclic path: a->b->d->a,
     * a->c->d->a. The current node matches d. Should return null.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_CyclicPath() throws Exception {
        String source = "<BreadCrumb><bc_node title=\"a\" url=\"/\" pattern=\"a\">"
            + "<bc_node title=\"b\" url=\"/\" pattern=\"b\"><bc_node title=\"d\" url=\"/\" pattern=\"d\">"
            + "<bc_node title=\"a\" url=\"/\" pattern=\"a\"/></bc_node></bc_node>"
            + "<bc_node title=\"c\" url=\"/\" pattern=\"c\"><bc_node title=\"d\" url=\"/\" pattern=\"d\"/>"
            + "</bc_node></bc_node></BreadCrumb>";

        pathDiscovery = new XmlPathDiscovery(source, false);

        assertNull("Returns an incorrect path", pathDiscovery.getPath(node, pageContext));
    }

    /**
     * Test of getPath method with partial cyclic path in the site map. Composes the site map as cyclic path:
     * a->b->d->a, a->c->d->a, e->c. The current node matches d. Should return the path as e->c->d. Also verifies if
     * the last element in the path is the current node.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_PartialCyclicPath() throws Exception {
        String source = "<BreadCrumb><bc_node title=\"a\" url=\"/\" pattern=\"a\">"
            + "<bc_node title=\"b\" url=\"/\" pattern=\"b\"><bc_node title=\"d\" url=\"/\" pattern=\"d\">"
            + "<bc_node title=\"a\" url=\"/\" pattern=\"a\"/></bc_node></bc_node>"
            + "<bc_node title=\"c\" url=\"/\" pattern=\"c\"><bc_node title=\"d\" url=\"/\" pattern=\"d\"/>"
            + "</bc_node></bc_node><bc_node title=\"e\" url=\"/\" pattern=\"e\">"
            + "<bc_node title=\"c\" url=\"/\" pattern=\"c\"/></bc_node></BreadCrumb>";

        pathDiscovery = new XmlPathDiscovery(source, false);

        BreadCrumbNode[] path = pathDiscovery.getPath(node, pageContext);
        String[] expected = {"e", "c", "d"};

        assertEquals("Returns an incorrect path", expected.length, path.length);

        for (int i = 0; i < (path.length - 1); i++) {
            assertEquals("Fails to set the url", "/", path[i].getUrl());
            assertEquals("Fails to set the title", expected[i], path[i].getTitle());
        }
        assertEquals("Fails to set the last node", node, path[path.length - 1]);
    }

    /**
     * Test of getPath method with non-cyclic path in the site map. Composes the site map as: a->b, a->c->d. The
     * current node matches b and d. Should return the path as a->b. Also verifies if the last element in the path is
     * the current node.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_NonCyclicPath() throws Exception {
        String source = "<BreadCrumb><bc_node title=\"a\" url=\"/\" pattern=\"a\">"
            + "<bc_node title=\"b\" url=\"/\" pattern=\"d\"/><bc_node title=\"c\" url=\"/\" pattern=\"c\">"
            + "<bc_node title=\"d\" url=\"/\" pattern=\"d\"/></bc_node></bc_node></BreadCrumb>";

        pathDiscovery = new XmlPathDiscovery(source, false);

        BreadCrumbNode[] path = pathDiscovery.getPath(node, pageContext);
        String[] expected = {"a", "b"};

        assertEquals("Returns an incorrect path", expected.length, path.length);

        for (int i = 0; i < (path.length - 1); i++) {
            assertEquals("Fails to set the url", "/", path[i].getUrl());
            assertEquals("Fails to set the title", expected[i], path[i].getTitle());
        }
        assertEquals("Fails to set the last node", node, path[path.length - 1]);
    }

    /**
     * Test of getPath method with matched current node having null title. Composes the site map as: a->b, a->c->d. The
     * current node matches b and d. Should return the path as a->b. Also verifies if the last element in the path has
     * the title from the node matcher b.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGetPath_OnNullTitle() throws Exception {
        // make sure the node has null title
        node = new BreadCrumbNode("d", null);

        String source = "<BreadCrumb><bc_node title=\"a\" url=\"/\" pattern=\"a\">"
            + "<bc_node title=\"b\" url=\"/\" pattern=\"d\"/><bc_node title=\"c\" url=\"/\" pattern=\"c\">"
            + "<bc_node title=\"d\" url=\"/\" pattern=\"d\"/></bc_node></bc_node></BreadCrumb>";

        pathDiscovery = new XmlPathDiscovery(source, false);

        BreadCrumbNode[] path = pathDiscovery.getPath(node, pageContext);
        String[] expected = {"a", "b"};
        int last = path.length - 1;

        assertEquals("Returns an incorrect path", expected.length, path.length);

        for (int i = 0; i < last; i++) {
            assertEquals("Fails to set the url", "/", path[i].getUrl());
            assertEquals("Fails to set the title", expected[i], path[i].getTitle());
        }

        // verify the last element - should have the node matcher's title
        assertEquals("Fails to set the url", node.getUrl(), path[last].getUrl());
        assertEquals("Fails to set the title", expected[last], path[last].getTitle());
    }
}
